@@ -1,3 +1,4 @@
+;;; -*- byte-compile-dynamic: t -*-
 ;;; mucs-ccl.el --- for MULE-UCS basic ccl configurations
 
 ;; Copyright (C) 1997-1999 Miyashita Hisashi
@@ -24,10 +25,8 @@
 
 ;; Comment:
 
-(require 'mucs)
 (require 'mucs-type)
 (require 'trans-util)
-(require 'tbl-mg)
 (require 'ccl)
 
 ;;; ------ Some notes on MUCS-CCL. ------
@@ -304,8 +303,7 @@
 (defsubst mucs-ccl-myo-from-ccl (ccl)
   (cons 'myo (list (list 'ccl ccl)
 		   (cons 'elisp nil)
-		   (cons 'table-set nil)))
-  )
+		   (cons 'table-set nil))))
 
 (defsubst mucs-ccl-myo-append (myo1 myo2)
   "Returns concatinated MYO1 and MYO2.
@@ -362,10 +360,9 @@ MYO1 and MYO2 may be destructed."
   (setcdr (assq 'table-set (cdr myo)) ccl))
 
 
-
-;;;
-;;;
-;;;
+;;
+;; conversion configuration handler
+;;
 
 (defun mucs-ccl-bind-program (progs)
   (let ((myo (mucs-ccl-empty-myo))
@@ -390,6 +387,10 @@ MYO1 and MYO2 may be destructed."
 	      '((repeat))))
      myo)))
 
+;;
+;; misc.
+;;
+
 (defun mucs-ccl-make-elisp-preparation-from-myo (myo)
   (append
    (mucs-ccl-myo-get-elisp myo)
@@ -397,14 +398,48 @@ MYO1 and MYO2 may be destructed."
     (function generate-table-set-registration-program)
     (mucs-ccl-myo-get-table-set myo))))
 
-(defun mucs-ccl-setup-ccl-program (mucs-ccl-prog)
-  (let ((mucs-current-type nil)
-	(mag (eval (car mucs-ccl-prog)))
-	(config1 (nth 1 mucs-ccl-prog))
-	(config2 (nth 2 mucs-ccl-prog))
-	(mucs-ccl-represent-2-form t)
+;;
+;; API.
+;;
+
+(defsubst mucs-convert (conversion object)
+  (let ((ccl-object
+	 (funcall
+	  (mucs-type-get-ccl-representation
+	   (car (mucs-conversion-get-conv-type conversion)))
+	  object))
+	(execute-vector
+	 (make-vector 8 0)))
+    (ccl-execute conversion
+		 (cond ((numberp ccl-object)
+			(aset execute-vector 0 ccl-object)
+			execute-vector)
+		       ((vectorp ccl-object)
+			ccl-object)
+		       (t
+			(signal 'mucs-ccl-invalid-error object))))
+    (if (= (aref execute-vector 0) mucs-invalid-code)
+	nil
+      (funcall
+       (mucs-type-get-elisp-representation
+	(cdr (mucs-conversion-get-conv-type conversion)))
+       (if (vectorp ccl-object)
+	   execute-vector
+	 (aref execute-vector 0))))))
+
+(defun mucs-setup-defun-convsersion (conv def)
+)
+
+(defun mucs-setup-conversion (conv def)
+  (let ((convtype (mucs-conversion-get-conv-type conv))
+	(mucs-current-type nil)
+	(mag (mucs-conversion-definition-mag def))
+	(config-main (mucs-conversion-definition-main-prog def))
+	(config-eof (mucs-conversion-definition-eof-prog def))
 	(main-ccl-myo (mucs-ccl-empty-myo))
 	(eof-ccl-myo (mucs-ccl-empty-myo))
+	ccl-program
+	compiled-ccl
 	func ret)
     (setq func (lambda (config myo)
 		 (while config
@@ -417,13 +452,30 @@ MYO1 and MYO2 may be destructed."
 		     (if ret
 			 (error "Invalid MUCS-CCL:%S" ret))))))
 
-    (funcall func config1 main-ccl-myo)
-    (funcall func config2 eof-ccl-myo)
-    (cons (append
-	   (mucs-ccl-make-elisp-preparation-from-myo main-ccl-myo)
-	   (mucs-ccl-make-elisp-preparation-from-myo eof-ccl-myo))
-	  (ccl-compile (list mag 
-			     (mucs-ccl-myo-get-ccl main-ccl-myo)
-			     (mucs-ccl-myo-get-ccl eof-ccl-myo))))))
+    (if (consp convtype)
+	(setq mucs-current-type (car convtype)))
+    (funcall func config-main main-ccl-myo)
+    (if (and (consp convtype)
+	     (not (eq (cdr convtype) mucs-current-type)))
+	(signal 'mucs-type-mismatch-error
+		(cons mucs-current-type (cdr convtype))))
+    (if (consp convtype)
+	(setq mucs-current-type (car convtype)))
+    (funcall func config-eof eof-ccl-myo)
+;;; EOF CCL program is not executed when non-stream conversion.
+;    (if (and (consp convtype)
+;	     (not (eq (cdr convtype) mucs-current-type)))
+;	(signal 'mucs-type-mismatch-error
+;		(cons mucs-current-type (cdr convtype))))
+    (setq ccl-program (list mag
+			    (mucs-ccl-myo-get-ccl main-ccl-myo)
+			    (mucs-ccl-myo-get-ccl eof-ccl-myo)))
+    (setq compiled-ccl (ccl-compile ccl-program))
+    (mucs-conversion-set-program-and-compiled-code
+     conv ccl-program compiled-ccl)
+    (append
+     (mucs-ccl-make-elisp-preparation-from-myo main-ccl-myo)
+     (mucs-ccl-make-elisp-preparation-from-myo eof-ccl-myo)
+     `((declare-ccl-program ,conv ,compiled-ccl)))))
 
 (provide 'mucs-ccl)
