@@ -43,7 +43,7 @@
 ;; 1.80->IMADEGAWA
 ;; 1.90->DAIRI
 ;; 2.00->Kyoto Univ.
-(defconst mucs-version "0.61(TOGETSUKYO:$BEO7n66(B)")
+(defconst mucs-version "0.62 (SHAKADO:$B<a2`F2(B)")
 
 ;;
 ;; For error handling.
@@ -66,6 +66,24 @@
 	    t)
       (require package)))
 
+(defun mucs-require-supplement (package &optional base)
+  "require supplement module."
+  (or (featurep package)
+      (if (or load-in-progress
+	      base)
+	  (load (expand-file-name 
+		 (symbol-name package)
+		 (file-name-directory
+		  (if (and (boundp 'load-file-name)
+			   (stringp 'load-file-name))
+		      load-file-name
+		    (if base
+			(locate-library
+			 (symbol-name base))
+		      (error "Cannot resolve the location of %s!"
+			     package))))))
+	(require package))))
+
 (defun mucs-max-code ()
   (1- (lsh 1 mucs-code-range-bits)))
 
